// Copyright (c) 2018, ETH Zurich and UNC Chapel Hill.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//
//     * Neither the name of ETH Zurich and UNC Chapel Hill nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// Author: Johannes L. Schoenberger (jsch at inf.ethz.ch)

#include <iostream>
#include <fstream>

#include "colmap/base/camera.h"

using namespace colmap;

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <pybind11/eigen.h>

namespace py = pybind11;

py::dict image_to_world(
        const std::vector<Eigen::Vector2d> points2D,
        const py::dict camera_dict
) {
    // Create camera.
    Camera camera;
    camera.SetModelIdFromName(camera_dict["model"].cast<std::string>());
    camera.SetWidth(camera_dict["width"].cast<size_t>());
    camera.SetHeight(camera_dict["height"].cast<size_t>());
    camera.SetParams(camera_dict["params"].cast<std::vector<double>>());

    // Image to world.
    std::vector<Eigen::Vector2d> world_points2D;
    for (size_t idx = 0; idx < points2D.size(); ++idx) {
        world_points2D.push_back(camera.ImageToWorld(points2D[idx]));
    }
    
    // Mean focal length.
    const double mean_focal_length = camera.MeanFocalLength();

    // Success output dictionary.
    py::dict success_dict;
    success_dict["world_points"] = world_points2D;
    success_dict["mean_focal_length"] = mean_focal_length;
    
    return success_dict;
}

py::dict world_to_image(
        const std::vector<Eigen::Vector2d> world_points2D,
        const py::dict camera_dict
) {
    // Create camera.
    Camera camera;
    camera.SetModelIdFromName(camera_dict["model"].cast<std::string>());
    camera.SetWidth(camera_dict["width"].cast<size_t>());
    camera.SetHeight(camera_dict["height"].cast<size_t>());
    camera.SetParams(camera_dict["params"].cast<std::vector<double>>());

    // World to image.
    std::vector<Eigen::Vector2d> image_points2D;
    for (size_t idx = 0; idx < world_points2D.size(); ++idx) {
        image_points2D.push_back(camera.WorldToImage(world_points2D[idx]));
    }
    
    // Success output dictionary.
    py::dict success_dict;
    success_dict["image_points"] = image_points2D;
    
    return success_dict;
}
